@@ -198,6 +198,18 @@ class Transformer(nn.Module):
 
         return null_logits + (logits - null_logits) * cond_scale
 
+    def forward_with_neg_prompt(
+        self,
+        text_embed: torch.Tensor,
+        neg_text_embed: torch.Tensor,
+        cond_scale = 3.,
+        **kwargs
+    ):
+        neg_logits = self.forward(*args, neg_text_embed = neg_text_embed, cond_drop_prob = 0., **kwargs)
+        pos_logits = self.forward(*args, text_embed = text_embed, cond_drop_prob = 0., **kwargs)
+
+        return neg_logits + (pos_logits - neg_logits) * cond_scale
+
     def forward(
         self,
         x,
@@ -218,7 +230,8 @@ class Transformer(nn.Module):
 
         if exists(texts):
             text_embeds = self.encode_text(texts)
-            context = self.text_embed_proj(text_embeds)
+
+        context = self.text_embed_proj(text_embeds)
 
         context_mask = (text_embeds != 0).any(dim = -1)
 
@@ -339,6 +352,7 @@ class MaskGit(nn.Module):
     def generate(
         self,
         texts: List[str],
+        negative_texts: Optional[List[str]] = None,
         cond_images: Optional[torch.Tensor] = None,
         fmap_size = None,
         temperature = 1.,
@@ -365,6 +379,20 @@ class MaskGit(nn.Module):
         starting_temperature = temperature
 
         cond_ids = None
+
+        text_embeds = self.transformer.encode_text(texts)
+
+        demask_fn = self.transformer.forward_with_cond_scale
+
+        # negative prompting, as in paper
+
+        neg_text_embeds = None
+        if exists(negative_texts):
+            assert len(texts) == len(negative_texts)
+
+            neg_text_embeds = self.transformer.encode_text(negative_texts)
+            demask_fn = partial(self.transformer.forward_with_neg_prompt, neg_text_embeds = neg_text_embeds)
+
         if self.resize_image_for_cond_image:
             assert exists(cond_images), 'conditioning image must be passed in to generate for super res maskgit'
             with torch.no_grad():
@@ -379,9 +407,9 @@ class MaskGit(nn.Module):
 
             ids = ids.scatter(1, masked_indices, self.mask_id)
 
-            logits = self.transformer.forward_with_cond_scale(
+            logits = demask_fn(
                 ids,
-                texts = texts,
+                text_embeds = text_embeds,
                 conditioning_token_ids = cond_ids,
                 cond_scale = cond_scale
             )
@@ -424,7 +452,8 @@ class MaskGit(nn.Module):
         self,
         images_or_ids: torch.Tensor,
         ignore_index = -1,
-        cond_images_or_ids: Optional[torch.Tensor] = None,
+        cond_images: Optional[torch.Tensor] = None,
+        cond_token_ids: Optional[torch.Tensor] = None,
         texts: Optional[List[str]] = None,
         text_embeds: Optional[torch.Tensor] = None,
         cond_drop_prob = None
@@ -454,17 +483,14 @@ class MaskGit(nn.Module):
 
         # tokenize conditional images if needed
 
-        cond_ids = None
+        assert not (exists(cond_images) and exists(cond_token_ids)), 'if conditioning on low resolution, cannot pass in both images and token ids'
 
-        if exists(cond_images_or_ids):
-            if cond_images_or_ids.dtype == torch.float:
-                assert exists(self.cond_vae), 'cond vqgan vae must be passed in'
-                assert all([height_or_width == self.cond_image_size for height_or_width in cond_images_or_ids.shape[-2:]])
+        if exists(cond_images):
+            assert exists(self.cond_vae), 'cond vqgan vae must be passed in'
+            assert all([height_or_width == self.cond_image_size for height_or_width in cond_images.shape[-2:]])
 
-                with torch.no_grad():
-                    _, cond_ids, _ = self.cond_vae.encode(cond_images_or_ids)
-            else:
-                cond_ids = cond_image_or_ids
+            with torch.no_grad():
+                _, cond_token_ids, _ = self.cond_vae.encode(cond_images)
 
         # prepare mask
 
@@ -487,7 +513,7 @@ class MaskGit(nn.Module):
             ids,
             texts = texts,
             text_embeds = text_embeds,
-            conditioning_token_ids = cond_ids,
+            conditioning_token_ids = cond_token_ids,
             labels = labels,
             cond_drop_prob = cond_drop_prob,
             ignore_index = ignore_index
